package ru.ulstu.extractor.ts.creator.db;

import org.springframework.stereotype.Component;
import ru.ulstu.extractor.author.model.Author;
import ru.ulstu.extractor.author.service.AuthorService;
import ru.ulstu.extractor.branch.model.Branch;
import ru.ulstu.extractor.branch.service.BranchService;
import ru.ulstu.extractor.commit.model.Commit;
import ru.ulstu.extractor.commit.service.CommitService;
import ru.ulstu.extractor.ts.model.TimeSeries;
import ru.ulstu.extractor.ts.model.TimeSeriesType;
import ru.ulstu.extractor.ts.model.TimeSeriesValue;
import ru.ulstu.extractor.ts.service.TimeSeriesService;

import java.util.ArrayList;
import java.util.List;

@Component
public class AuthorsCommitTS extends DBTimeSeriesCreator {
    private final TimeSeriesService timeSeriesService;
    private final CommitService commitService;
    private final AuthorService authorService;
    private final BranchService branchService;

    public AuthorsCommitTS(TimeSeriesService timeSeriesService,
                           CommitService commitService,
                           AuthorService authorService,
                           BranchService branchService) {
        this.timeSeriesService = timeSeriesService;
        this.commitService = commitService;
        this.authorService = authorService;
        this.branchService = branchService;
    }

    @Override
    public TimeSeriesType getTimeSeriesType() {
        return TimeSeriesType.AUTHOR_COMMITS;
    }

    @Override
    public List<TimeSeries> getTimeSeries(Branch branch) {
        List<TimeSeries> timeSeriesResult = new ArrayList<>();
        List<Author> authors = authorService.findAll();
        List<Commit> commits = new ArrayList<>(commitService.findByRepositoryIdAndName(branch.getGitRepository().getId(),
                branch.getName()));
        for (Author author : authors) {
            TimeSeries timeSeries = new TimeSeries(
                    String.format("%s %s %s %s",
                            branch.getGitRepository().getName(),
                            branch.getName(),
                            author.getName(),
                            getTimeSeriesType().getDescription()),
                    branchService.findByRepositoryAndName(branch.getGitRepository(),
                            branch.getName()),
                    getTimeSeriesType());
            for (Commit commit : commits) {
                if (commit.getAuthor().equals(author)) {
                    timeSeries.getValues().add(new TimeSeriesValue(commit.getDate(), 1.0));
                }
            }
            if (!timeSeries.getValues().isEmpty()) {
                timeSeriesResult.add(timeSeries);
            }
        }
        return timeSeriesResult;
    }

    @Override
    public TimeSeriesService getTimeSeriesService() {
        return timeSeriesService;
    }
}
